unit U_Config;

interface

uses IniFiles, Sysutils, U_SourceCodeFormatter_KRL, U_Basic, U_Constants;

function LoadConfig: Boolean;

{$IFDEF WithGUI}
function LoadConfig_GUI: Boolean;
function SaveConfig_GUI: Boolean;
{$ENDIF}

var
  CFG_SpacingOperators: boolean;
  CFG_IndentWidth: Integer;

  CFG_CAP_ProgramFlowStrings, CFG_CAP_SetupCommandStrings, CFG_CAP_MotionCommandStrings,
    CFG_CAP_MathAndFunctionStrings, CFG_CAP_IOCommandStrings, CFG_CAP_TypeDefinitionStrings,
    CFG_CAP_SystemVariableStrings, CFG_CAP_OpusTechFunctionStrings: TCapitalizationMode;

implementation

{$IFDEF WithGUI}
uses Form_Main;
{$ENDIF}


{==============================================================================}

function GetConfigFileName: string;
var Path: string;
begin
  Path := ExtractFilePath(ParamStr(0));
  if (Path <> '') and (Path[length(Path)] <> '\') then Path := Path + '\';
  Result := Result + Path + IniFileName;
end;
{==============================================================================}

function ConfigFileExists: Boolean;
begin
  Result := FileExists(GetConfigFileName);
end;
{==============================================================================}
{$IFDEF WithGUI}

function LoadConfig_GUI: Boolean;
  function CapitalizationModeToItemIndex(CapMode: TCapitalizationMode): Integer;
  begin
    case CapMode of
      capUpperCase: Result := 1;
      capLowerCase: Result := 2;
    else Result := 0;
    end;
  end;

begin
  Result := LoadConfig;
  with KRLSCF_MainForm do
  begin
    CheckBox_SpacingOperators.Checked := CFG_SpacingOperators;
    Spin_IdentWidth.Value := CFG_IndentWidth;
    ComboBox_ProgramFlowStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_ProgramFlowStrings);
    ComboBox_SetupCommandStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_SetupCommandStrings);
    ComboBox_MotionCommandStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_MotionCommandStrings);
    ComboBox_MathAndFunctionStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_MathAndFunctionStrings);
    ComboBox_IOCommandStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_IOCommandStrings);
    ComboBox_TypeDefinitionStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_TypeDefinitionStrings);
    ComboBox_SystemVariableStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_SystemVariableStrings);
    ComboBox_OpusTechFunctionStrings.ItemIndex := CapitalizationModeToItemIndex(CFG_CAP_OpusTechFunctionStrings)
  end;
end;
{==============================================================================}

function SaveConfig_GUI: Boolean;
var TheIni: TIniFile;

  function ItemIndexToCapitalizationMode(ItemIndex: Integer): TCapitalizationMode;
  begin
    case ItemIndex of
      1: Result := capUpperCase;
      2: Result := capLowerCase;
    else Result := capNone;
    end;
  end;

  procedure WriteCapitalizationMode(Section, Ident: string; CapMode: TCapitalizationMode);
  begin
    case CapMode of
      capUpperCase: TheIni.WriteString(Section, Ident, 'Uppercase');
      capLowerCase: TheIni.WriteString(Section, Ident, 'Lowercase');
    else TheIni.WriteString(Section, Ident, 'None');
    end;
  end;

begin
  Result := True;
  with KRLSCF_MainForm do
  begin
    CFG_SpacingOperators := CheckBox_SpacingOperators.Checked;
    CFG_IndentWidth := Spin_IdentWidth.Value;
    CFG_CAP_ProgramFlowStrings := ItemIndexToCapitalizationMode(ComboBox_ProgramFlowStrings.ItemIndex);
    CFG_CAP_SetupCommandStrings := ItemIndexToCapitalizationMode(ComboBox_SetupCommandStrings.ItemIndex);
    CFG_CAP_MotionCommandStrings := ItemIndexToCapitalizationMode(ComboBox_MotionCommandStrings.ItemIndex);
    CFG_CAP_MathAndFunctionStrings := ItemIndexToCapitalizationMode(ComboBox_MathAndFunctionStrings.ItemIndex);
    CFG_CAP_IOCommandStrings := ItemIndexToCapitalizationMode(ComboBox_IOCommandStrings.ItemIndex);
    CFG_CAP_TypeDefinitionStrings := ItemIndexToCapitalizationMode(ComboBox_TypeDefinitionStrings.ItemIndex);
    CFG_CAP_SystemVariableStrings := ItemIndexToCapitalizationMode(ComboBox_SystemVariableStrings.ItemIndex);
    CFG_CAP_OpusTechFunctionStrings := ItemIndexToCapitalizationMode(ComboBox_OpusTechFunctionStrings.ItemIndex);
  end;

  TheIni := TIniFile.Create(GetConfigFileName);
  try
    try
      TheIni.WriteBool('Spacing', 'SpacingOperators', CFG_SpacingOperators);
      TheIni.WriteInteger('Indent', 'IndentWidth', CFG_IndentWidth);

      WriteCapitalizationMode('Capitalization', 'CAP_ProgramFlowStrings', CFG_CAP_ProgramFlowStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_SetupCommandStrings', CFG_CAP_SetupCommandStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_MotionCommandStrings', CFG_CAP_MotionCommandStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_MathAndFunctionStrings', CFG_CAP_MathAndFunctionStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_IOCommandStrings', CFG_CAP_IOCommandStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_TypeDefinitionStrings', CFG_CAP_TypeDefinitionStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_SystemVariableStrings', CFG_CAP_SystemVariableStrings);
      WriteCapitalizationMode('Capitalization', 'CAP_OpusTechFunctionStrings', CFG_CAP_OpusTechFunctionStrings);
    except
      on E: Exception do
      begin
        Result := False;
        WriteLn(Format('Error: Exception while reading from config file "%s". Message: %s', [GetConfigFileName, E.Message]));
      end;
    end;
  finally
    TheIni.Free;
  end;



end;
{$ENDIF}

{==============================================================================}

function LoadConfig: Boolean;
var TheIni: TIniFile;

  function ReadCapitalizationMode(Section, Ident: string): TCapitalizationMode;
  begin
    if StrCompare(TheIni.ReadString(Section, Ident, 'None'), 'UpperCase') then Result := capUpperCase
    else if StrCompare(TheIni.ReadString(Section, Ident, 'None'), 'LowerCase') then Result := capLowerCase
    else Result := capNone;
  end;

begin
  Result := True;

  if ConfigFileExists then
  begin
    TheIni := TIniFile.Create(GetConfigFileName);
    try
      try
        CFG_SpacingOperators := TheIni.ReadBool('Spacing', 'SpacingOperators', true);
        CFG_IndentWidth := TheIni.ReadInteger('Indent', 'IndentWidth', 2);

        CFG_CAP_ProgramFlowStrings := ReadCapitalizationMode('Capitalization', 'CAP_ProgramFlowStrings');
        CFG_CAP_SetupCommandStrings := ReadCapitalizationMode('Capitalization', 'CAP_SetupCommandStrings');
        CFG_CAP_MotionCommandStrings := ReadCapitalizationMode('Capitalization', 'CAP_MotionCommandStrings');
        CFG_CAP_MathAndFunctionStrings := ReadCapitalizationMode('Capitalization', 'CAP_MathAndFunctionStrings');
        CFG_CAP_IOCommandStrings := ReadCapitalizationMode('Capitalization', 'CAP_IOCommandStrings');
        CFG_CAP_TypeDefinitionStrings := ReadCapitalizationMode('Capitalization', 'CAP_TypeDefinitionStrings');
        CFG_CAP_SystemVariableStrings := ReadCapitalizationMode('Capitalization', 'CAP_SystemVariableStrings');
        CFG_CAP_OpusTechFunctionStrings := ReadCapitalizationMode('Capitalization', 'CAP_OpusTechFunctionStrings');
      except
        on E: Exception do
        begin
          Result := False;
          WriteLn(Format('Error: Exception while reading from config file "%s". Message: %s', [GetConfigFileName, E.Message]));
        end;
      end;
    finally
      TheIni.Free;
    end;
  end else Result := False;
end;
{==============================================================================}

procedure InitConfigVars;
begin
  CFG_IndentWidth := 2;
  CFG_SpacingOperators := True;

  CFG_CAP_ProgramFlowStrings := capUpperCase;
  CFG_CAP_SetupCommandStrings := capUpperCase;
  CFG_CAP_MotionCommandStrings := capUpperCase;
  CFG_CAP_MathAndFunctionStrings := capUpperCase;
  CFG_CAP_IOCommandStrings := capUpperCase;
  CFG_CAP_TypeDefinitionStrings := capUpperCase;
  CFG_CAP_SystemVariableStrings := capUpperCase;
  CFG_CAP_OpusTechFunctionStrings := capNone;
end;
{==============================================================================}
initialization
  InitConfigVars;

end.

