unit U_SourceCodeFormatter_KRL;

interface

uses

{$IFDEF WithGUI}
  Dialogs,
{$ENDIF}
  sysutils,
  classes, U_Basic;

type TCapitalizationMode = (capNone, capUpperCase, capLowerCase);

procedure FormatSource(FileName: string);

implementation

uses U_Config;

const
  IndentStrings: array[0..10] of string = ('def', 'deffct', 'DEFDAT', 'if',
    'loop', 'repeat', 'for', ';fold', 'switch', 'while', 'else');

  UnindentStrings: array[0..10] of string = ('endfor', 'endif', 'endwhile',
    'endloop', 'end', 'ENDDAT', ';endfold', 'endfct', 'endswitch', 'else', 'until');

  IndentStringsCase: array[0..2] of string = ('SWITCH', 'case', 'default');

  UnindentStringsCase: array[0..2] of string = ('case', 'default', 'endswitch');

  Delimeters: array[0..22] of string = ('(', ',', '-', '~', '*', '(', ')', '|',
    '{', '}', #9, '[', ']', '<', '>', '=', '"', '''', ' ', ',', '?', ':', '+');

  Quotes: array[0..1] of string = ('"', '''');

  ProgramFlowStrings: array[0..36] of string = (
    'continue', 'case', 'decl', 'def', 'do', 'defdat', ';endfold', 'default', 'deffct',
    'endfor', 'enum', 'endif', 'endwhile', 'else', 'endloop', 'end', 'endfct',
    'enddat', 'endswitch', 'ext', 'exit', 'for', ';fold', 'halt', 'if', 'loop',
    'prio', 'repeat', 'return', 'resume', 'switch', 'step', 'then', 'to',
    'until', 'while', 'when');

  SetupCommandStrings: array[0..24] of string = (
    '$pro_act', '$CONF_MESS', 'A', 'base', 'base_no', 'B', 'cp_dat', 'C',
    'ex_base', 'ERR_TO_PLC', 'fdat', 'frames', 'group', 'interrupt',
    'ipo_frame', 'ldat', 'out_sync', 'out_async', 'odat', 'sec', 'tool_no',
    'tool', 'X', 'Y', 'Z');

  MotionCommandStrings: array[0..17] of string = (
    '$POS_ACT', '#fast', '#normal', '#slow', '#veryslow', '#medium',
    '#extraslow', '#path', 'brake', 'circ', 'c_ptp', 'c_vel', 'c_dis',
    'c_ori', 'lin', 'lin_rel', 'ptp', 'ptp_rel');

  MathAndFunctionStrings: array[0..18] of string = (
    'and', 'abs', 'acos', 'atan2', 'b_and', 'b_exor', 'b_not', 'b_or',
    'cos', 'exor', 'false', 'not', 'or', 'off', 'on', 'sin', 'sqrt',
    'tan', 'true');

  IOCommandStrings: array[0..9] of string = (
    '#open_gr', '#close_gr', 'distance', 'delay', 'mess', 'pulse', 'path',
    'signal', 'trigger', 'wait');

  TypeDefinitionStrings: array[0..9] of string = (
    'axis', 'bool', 'char', 'e6pos', 'e6axis', 'frame', 'int', 'pos',
    'real', 'struc');

  SystemVariableStrings: array[0..8] of string = (
    '$ipo_mode', '$out', '$in', '$timer_stop', '$cycflag', '$timer',
    '$flag', '$mode_op', '$MSG_T');

  OpusTechFunctionStrings: array[0..8] of string = (
    'appro', 'appros', 'depart', 'departs', 'errorout', 'linspeed',
    'newtool', 'newbase', 'setspeed');

{==============================================================================}

procedure MyWriteLn(Line: string);
begin
{$IFNDEF WithGUI}
  WriteLn(Line);
{$ENDIF}
end;
{==============================================================================}


function StringInArray(Str: string; Arr: array of string): Boolean;
var i: Integer;
begin
  Result := False;
  for i := low(Arr) to high(Arr) do
    if StrCompare(Arr[i], Str) then
    begin
      Result := True;
      break;
    end;
end;

{==============================================================================}

function FormatWords(Line: string; Arr: array of string; Mode: TCapitalizationMode): string;
var i: Integer;
  sWord: string;
begin
  sWord := '';
  for i := 1 to length(Line) do
  begin
    if StringInArray(Line[i], Delimeters) then
    begin
      break
    end
    else sWord := sWord + Line[i];
  end;
end;
{==============================================================================}

function PosIsInComment(Offset: Integer; Str: string): boolean;
var P: Integer;
const CommentSign = ';';
begin
  Result := False;
  P := Pos(CommentSign, Str);
  if P > 0 then Result := (P < Offset);
end;
{==============================================================================}

function PosIsInQuotes_Ex(Offset: Integer; Str, QuoteSign: string): boolean;
var i, n: Integer;
begin
  n := 0;
  for i := 1 to Offset do
  begin
    if Offset > Length(Str) then break;
    if StrCompare(Str[i], QuoteSign) then inc(n);
  end;

  Result := (n mod 2) <> 0;
end;

function PosIsInQuotes(Offset: Integer; Str: string): boolean;
var i: Integer;
begin
  Result := False;
  for i := low(Quotes) to high(Quotes) do
    if PosIsInQuotes_Ex(Offset, Str, Quotes[i]) then
    begin
      Result := True;
      break;
    end;
end;

function PosIsWord(Offset, Size: Integer; Str: string): boolean;
var LeftIsDelimeter, RightIsDelimeter: Boolean;
begin
  if Offset <= 1 then LeftIsDelimeter := True
  else LeftIsDelimeter := StringInArray(Str[Offset - 1], Delimeters);

  if Offset + Size >= Length(Str) then RightIsDelimeter := True
  else RightIsDelimeter := StringInArray(Str[Offset + Size], Delimeters);

  Result := RightIsDelimeter and LeftIsDelimeter;
end;
{==============================================================================}

function MyStringReplace(const S, OldPattern, NewPattern: string; Flags: TReplaceFlags; const OnlyReplaceWords: Boolean = False): string;
var
  SearchStr, SearchStrOrig, Patt, NewStr: string;
  Offset: Integer;
  DoReplace: Boolean;
begin
  if rfIgnoreCase in Flags then
  begin
    SearchStr := AnsiUpperCase(S);
    SearchStrOrig := S;
    Patt := AnsiUpperCase(OldPattern);
  end else
  begin
    SearchStr := S;
    SearchStrOrig := S;
    Patt := OldPattern;
  end;
  NewStr := S;
  Result := '';
  while SearchStr <> '' do
  begin
    Offset := AnsiPos(Patt, SearchStr);

    if Offset = 0 then
    begin
      Result := Result + NewStr;
      Break;
    end;

    if (PosIsInComment(Offset + (length(S) - length(SearchStr)), S)) or
      (PosIsInQuotes(Offset + (length(S) - length(SearchStr)), S)) or
      ((OnlyReplaceWords) and (not PosIsWord(Offset + (length(S) - length(SearchStr)), length(OldPattern), S))) then
      DoReplace := False
    else DoReplace := True;


    if DoReplace then Result := Result + Copy(NewStr, 1, Offset - 1) + NewPattern
    else
    begin
      Result := Result + Copy(NewStr, 1, Offset - 1) + Copy(SearchStrOrig, Offset, length(OldPattern));
    end;

    NewStr := Copy(NewStr, Offset + Length(OldPattern), MaxInt);
    if not (rfReplaceAll in Flags) then
    begin
      Result := Result + NewStr;
      Break;
    end;
    SearchStr := Copy(SearchStr, Offset + Length(Patt), MaxInt);
    SearchStrOrig := Copy(SearchStrOrig, Offset + Length(Patt), MaxInt);
  end;
end;
{==============================================================================}

function ReplaceWords(Line: string; Arr: array of string; Mode: TCapitalizationMode): string;
var i: Integer;
begin
  Result := Line;
  if Mode = capNone then Exit;

  for i := low(Arr) to high(Arr) do
  begin
    case Mode of
      capUpperCase: Result := MyStringReplace(Result, Arr[i], ANSIUpperCase(Arr[i]), [rfReplaceAll, rfIgnoreCase], true);
      capLowerCase: Result := MyStringReplace(Result, Arr[i], ANSILowerCase(Arr[i]), [rfReplaceAll, rfIgnoreCase], true);
    end;
  end;
end;
{==============================================================================}

function SpacingOperators(Line: string): string;
var NewLine, OldLine: string;
begin
  NewLine := Line;


  NewLine := MyStringReplace(NewLine, '>=', #242, [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '<=', #243, [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '<>', #247, [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '==', #240, [rfReplaceAll]);

  NewLine := MyStringReplace(NewLine, '=', ' = ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '*', ' * ', [rfReplaceAll]);
  // NewLine := MyStringReplace(NewLine, '+', ' + ', [rfReplaceAll]);
  // NewLine := MyStringReplace(NewLine, '-', ' - ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '/', ' / ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '>', ' > ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, '<', ' < ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, ',', ', ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, ';', ' ;', [rfReplaceAll]);

  // NewLine := MyStringReplace(NewLine, ':', ' : ', [rfReplaceAll]);

  NewLine := MyStringReplace(NewLine, #242, ' >= ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, #243, ' <= ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, #247, ' <> ', [rfReplaceAll]);
  NewLine := MyStringReplace(NewLine, #240, ' == ', [rfReplaceAll]);

  NewLine := MyStringReplace(NewLine, ')OR(', ') OR (', [rfReplaceAll, rfIgnoreCase]);
  NewLine := MyStringReplace(NewLine, ') OR(', ') OR (', [rfReplaceAll, rfIgnoreCase]);
  NewLine := MyStringReplace(NewLine, ')OR (', ') OR (', [rfReplaceAll, rfIgnoreCase]);

  NewLine := MyStringReplace(NewLine, ')AND(', ') AND (', [rfReplaceAll, rfIgnoreCase]);
  NewLine := MyStringReplace(NewLine, ') AND(', ') AND (', [rfReplaceAll, rfIgnoreCase]);
  NewLine := MyStringReplace(NewLine, ')AND (', ') AND (', [rfReplaceAll, rfIgnoreCase]);

  while (Pos('  ', NewLine) <> 0) do
  begin
    OldLine := NewLine;
    NewLine := MyStringReplace(NewLine, '  ', ' ', [rfReplaceAll]);

    if OldLine = NewLine then break;
  end;

  Result := NewLine;
end;
{==============================================================================}

function GetFirstWord(Line: string): string;
var i: Integer;
begin
  Result := '';
  for i := 1 to length(Line) do
  begin
    if StringInArray(Line[i], Delimeters) then break
    else Result := Result + Line[i];
  end;
end;
{==============================================================================}

function GetIndentStr(Width: Integer): string;
var n: Integer;
const IndentChar = ' ';
begin
  Result := '';
  for n := 1 to Width do Result := Result + IndentChar;
end;
{==============================================================================}

function LoadFile(lFileName: string; lFileText: TStrings): boolean;
var errmsg: string;
begin
  Result := True;

  MyWriteLn(Format('Debug: Open file "%s".', [lFileName]));

  if FileExists(lFileName) then
  begin
    try
      lFileText.LoadFromFile(lFileName);
    except
      on E: Exception do
      begin
        Result := False;
        errmsg := Format('Can not open file "%s". Message: %s', [lFileName, E.Message]);

{$IFDEF WithGUI}
        MessageDlg(errmsg, mtError, [mbOK], 0);
{$ENDIF}
        MyWriteLn('Error: ' + errmsg);
      end;
    end;
  end
  else
  begin
    Result := False;
    errmsg := 'File "%s" do not exist!';

{$IFDEF WithGUI}
    MessageDlg(errmsg, mtError, [mbOK], 0);
{$ENDIF}
    MyWriteLn('Error: ' + errmsg);
  end;
end;
{==============================================================================}

function SaveFile(sFileName: string; sFileText: TStrings): boolean;
var errmsg: string;
begin
  Result := True;
  MyWriteLn(Format('Debug: Save file "%s".', [sFileName]));

  try
    sFileText.SaveToFile(sFileName);
  except
    on E: Exception do
    begin
      Result := False;
      errmsg := Format('Can not save file "%s". Message: %s', [sFileName, E.Message]);

{$IFDEF WithGUI}
      MessageDlg(errmsg, mtError, [mbOK], 0);
{$ENDIF}
      MyWriteLn('Error: ' + errmsg);
    end;
  end;
end;
{==============================================================================}

procedure FormatSource(FileName: string);
var
  FileText: TStrings;
  i, Indent, IndentCase: Integer;
  Line, NewLine: string;

begin
  FileText := TStringList.Create;
  try
    if LoadFile(FileName, FileText) and SaveFile(FileName + '~', FileText) then
    begin
      MyWriteLn(Format('Debug: Process file "%s".', [FileName]));
      Indent := 0;
      IndentCase := 0;
      for i := 0 to FileText.Count - 1 do
      begin
        Line := FileText[i];

        if CFG_SpacingOperators then Line := SpacingOperators(Line);

        Line := ReplaceWords(Line, ProgramFlowStrings, CFG_CAP_ProgramFlowStrings);
        Line := ReplaceWords(Line, SetupCommandStrings, CFG_CAP_SetupCommandStrings);
        Line := ReplaceWords(Line, MotionCommandStrings, CFG_CAP_MotionCommandStrings);
        Line := ReplaceWords(Line, MathAndFunctionStrings, CFG_CAP_MathAndFunctionStrings);
        Line := ReplaceWords(Line, IOCommandStrings, CFG_CAP_IOCommandStrings);
        Line := ReplaceWords(Line, TypeDefinitionStrings, CFG_CAP_TypeDefinitionStrings);
        Line := ReplaceWords(Line, SystemVariableStrings, CFG_CAP_SystemVariableStrings);
        Line := ReplaceWords(Line, OpusTechFunctionStrings, CFG_CAP_OpusTechFunctionStrings);

        Line := TrimLeft(Line);


        if StringInArray(GetFirstWord(Line), UnindentStrings) then inc(Indent, -1);

        if StringInArray(GetFirstWord(Line), UnindentStringsCase) then inc(IndentCase, -1);
        if IndentCase < 0 then IndentCase := 0;

        NewLine := { IntToStr(Indent) + ' ' + IntToStr(IndentCase) + ' ' + } GetIndentStr(CFG_IndentWidth * Indent) + GetIndentStr(CFG_IndentWidth * IndentCase) + Line;

        if StringInArray(GetFirstWord(Line), IndentStrings) then inc(Indent, 1);
        if StringInArray(GetFirstWord(Line), IndentStringsCase) then inc(IndentCase, 1);

        FileText[i] := NewLine;
      end;

      SaveFile(FileName, FileText);

      MyWriteLn(Format('Debug: Done file "%s".', [FileName]));
    end;
  finally
    FileText.Free;
  end;
end;

end.

